describe WechatClient::Client do
  it "get client" do
    client = WechatClient.client
    expect(client.options).to be_a(OpenStruct)
    expect(client.options.app_id.empty?).to eq false
    expect(client.options.verify_ssl).to eq false
  end

  it "get cached access_token" do
    at = WechatClient.client.get(:access_token){|json| json['access_token']}
    at_ag = WechatClient.client.access_token{|json| json['access_token']}
    expect(at.is_a?(String)).to eq true
    expect(at).to eq at_ag
  end

  it "get uncached access_token" do
    options = WechatClient.options.merge(auto_cache: false)
    client = WechatClient::Client.new options
    at = client.get(:access_token){|json| json['access_token']}
    at_ag = client.access_token{|json| json['access_token']}
    expect(at == at_ag).to eq false
  end

  it "force get new access_token" do
    at_ag = WechatClient.client.get(:__access_token__){|json| json['access_token']}
    at = WechatClient.client.get(:access_token){|json| json['access_token']}
    WechatClient.cache.delete("__wechat_client_access_token__")
    expect(at == at_ag).to eq false
  end

  # it "post message" do
  #   message = {
  #     touser: 'o7c6JwXe9EqzIXyMZDWAFHn_Dzps',
  #     msgtype: 'text',
  #     text: {content: 'hello world'}
  #   }
  #   errcode = WechatClient.client.post(:send_message, message) {|json| json["errcode"]}
  #   expect(errcode).to eq 0
  # end

  it "mulit threads" do
    options = WechatClient.options.merge(auto_cache: true)
    client = WechatClient::Client.new options
    threads = []
    result = ThreadSafe::Array.new

    RestClient::Request.stub(:execute).and_return("test")
    10.times do
      100.times do
        threads << Thread.new do
          client.get(:access_token)
          result << client.url
        end
      end
      100.times do
        threads << Thread.new do
          client.get(:jsapi_ticket)
          result << client.url
        end
      end
      threads.each(&:join)
      count = result.select{|url| url == WechatClient::Api.new(client.options).access_token}.count
      expect(count).to eq 100
      result.clear
    end
  end

  it "get base info" do
    openid = "oCo5Tt6gj2kghPQ4_avdQd4nwlMA"
    unionid = WechatClient.client.base_info openid: openid do |json|
      json["unionid"]
    end
    expect(unionid).to eq "oHv8ewZPGaZokH2ScUkWxiSGKY7s"
  end
end
